/*+++
 *  title:      ModalPatch.m
 *  abstract:   Implementation of ModalPatch InputManager for (OS|YB)/Windows.
 *  author:     Tom Hageman <tom@basil.icce.rug.nl>
 *  created:    February 1999, from an original hacque by Jeremy Bettis. 
 *  modified:   
 *  copyleft:
 *
 *		Copyright (C) 1997  Jeremy Bettis, St\'ephane Corth\'esy (Patches)
 *		Copyright (C) 1999  Tom R. Hageman. (Integration)
 *
 *	This is free software.
 *
 *	This software is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *  description:
 *      (see corresponding *.h file)
 *---*/

#import "ModalPatch.h"
#import "ModalPatchApplication.h"

/* The following aren't really ModalWindow related patches,
   but are thrown in at no extra cost :-) */

#if defined(WIN32) && !defined(PAGELAYOUT_PATCH)
#   define PAGELAYOUT_PATCH 1
#endif

// ====================================================================
// A NSForm inside of a NSScrollView does not handle mouse clicks
// correctly. This subclass fixes that.  It scrolls the old selected
// cell to visible while activating the new one.
// [TRH] (This bugfix seems valid for both OS/Win and OS/Mach 4.2;
//  maybe it is fixed in Rhapsody/MacOSX(S) and YB/Win -- I've added a
//  conditional on event type, so the workaround only kicks in on mouse-down
//  clicks.  This turned out to be necessary in order to retain auto-scroll
//  when tabbing through subsequent form fields.)
// ====================================================================

static int dontdoit = 0;

@interface ModalPatchMatrix : NSMatrix
+ (void)enableMatrixScrollPatch; 
- (void)scrollCellToVisibleAtRow:(int)r column:(int)c;
- (void)textDidEndEditing:(NSNotification *)notify;
@end


@implementation ModalPatchMatrix

+ (void)enableMatrixScrollPatch
{
    if (self != [NSMatrix class])
    {
        [self poseAsClass:[NSMatrix class]];
    }
}

- (void)scrollCellToVisibleAtRow:(int)r column:(int)c
{
    if (dontdoit <= 0)
    {
        [super scrollCellToVisibleAtRow:r column:c];
    }
}

- (void)textDidEndEditing:(NSNotification *)notify
{
    NSEventType eventType = [[[NSApplication sharedApplication] currentEvent] type];

    if (NSEventMaskFromType(eventType) & (NSLeftMouseDownMask | NSRightMouseDownMask))
    {
        dontdoit++;
        NS_DURING
            [super textDidEndEditing:notify];
        NS_HANDLER
            dontdoit--;
            [localException raise];
        NS_ENDHANDLER
        dontdoit--;
    }
    else
    {
        [super textDidEndEditing:notify];
    }
}

@end

// ======================================================================

#if PAGELAYOUT_PATCH

/* Here's another hack, to correct a problem of -[NSApplication
   runPageLayout:] on OS/NT 4.1: after that call, the pageLayout panel is
   popped up, and the user can set the layout; after he dismissed the
   panel with the OK button, the method +[NSPrintInfo sharedPrintInfo]
   returns nil, whatever the user chose.
   By St\'ephane Corth\'esy <stephane@lysis.ch>
   [TRH] This may or may not have been corrected in OSE4.2 and/or YB/WO4.0. */
#import <AppKit/NSPrintInfo.h>
#import <AppKit/NSPageLayout.h>

@implementation ModalPatchApplication (PageLayoutPatch) 

- (void) runPageLayout:(id)sender 
{ 
        NSPrintInfo *tempPrintInfo = [[NSPrintInfo sharedPrintInfo] copy]; 
        NSPageLayout *pageLayout = [NSPageLayout pageLayout]; 
        int runResult; 

        runResult = [pageLayout runModalWithPrintInfo:tempPrintInfo]; 

        if(runResult == NSOKButton) 
                [NSPrintInfo setSharedPrintInfo:tempPrintInfo]; 
        [tempPrintInfo release]; 
} 

@end 

#endif // PAGELAYOUT_PATCH

// ======================================================================


@implementation ModalPatch

+ (void)load
{
    [self class]; // Causes +initialize to fire, which does additional setup.
}

+ (void)initialize
{
    NSUserDefaults *userDefaults;

    if (self != [ModalPatch class]) return;

    userDefaults = [NSUserDefaults standardUserDefaults];

    [userDefaults registerDefaults:
        [[[NSBundle bundleForClass:self] infoDictionary]
            objectForKey:@"ModalPatchDefaults"]];

    if ([userDefaults boolForKey:@"ModalPatchEnable"])
    {
        [self enableModalPatch];
    }
    if ([userDefaults boolForKey:@"MatrixScrollPatchEnable"])
    {
        [self enableMatrixScrollPatch];
    }
}

+ (void)enableModalPatch
{
    [ModalPatchApplication enableModalPatch];
}

+ (void)enableMatrixScrollPatch
{
    [ModalPatchMatrix enableMatrixScrollPatch];
}

@end // ModalPatch
