/*+++
 *  title:      ModalPatchApplication.m
 *  abstract:   Implementation of ModalPatchApplication class for (OS|YB)/Windows.
 *  author:     Tom Hageman <tom@basil.icce.rug.nl>
 *  created:    February 1999, from an original hacque by Jeremy Bettis. 
 *  modified:   
 *  copyleft:
 *
 *    Original  Copyright (C) 1997  Jeremy Bettis
 * Adaptations  Copyright (C) 1999  Tom R. Hageman.
 *
 *	This is free software.
 *
 *	This software is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 *  description:
 *      (see corresponding *.h file)
 *---*/

#import "ModalPatchApplication.h"

#if defined(WIN32) && !defined(MODAL_PATCH)
#   define MODAL_PATCH 1
#endif

// ====================================================================
// Modal windows can be lost when the user uses alt-tab to switch from
// one window to another.  This subclass of NSApplication forces the
// modal window to always be in front.
// Unfortunately this patch does not work for native WinXX modal windows
// like the open- and save panel :-(
// ====================================================================

@interface NSApplication (ModalPatchUndocumentedFunctions)
- (BOOL)_isRunningModal;
- _orderFrontModalWindow:(NSWindow *)aWindow;
@end


/* Support function to transmogrify class of existing instance. */
static void changeClassOfInstanceTo(id instance, Class class)
{
    /* Don't try this at home, kids */
    if (instance) instance->isa = class;
}


@implementation ModalPatchApplication

+ (void)enableModalPatch
{
#if MODAL_PATCH
    if (self != [NSApplication class])
    {
        NSApplication *theApp;
        Class oldClass = [NSApplication class];

        [self poseAsClass:oldClass];
        theApp = [NSApplication sharedApplication];
        // Uggg.  Force existing NSApp instance to posing class.
	// (But only if its class is exactly NSApplication)
        if ([theApp class] == oldClass)
        {
            changeClassOfInstanceTo(theApp, self);
        }
        [[NSNotificationCenter defaultCenter] addObserver:theApp
            selector:@selector(modalPatchWindowDidBecomeKey:)
            name:NSWindowDidBecomeKeyNotification object:nil];
    }
#endif
}

#if MODAL_PATCH


// Private support functions for modal-window stack.

NSMutableArray *_modalWindows;

static void addModalWindow(NSWindow *aWindow)
{
    if (_modalWindows == nil)
    {
        _modalWindows = [[NSMutableArray alloc] initWithCapacity:1];
    }
    [_modalWindows addObject:aWindow];
}

static void removeLastModalWindow()
{
    if (_modalWindows && [_modalWindows count] > 0)
    {
        [_modalWindows removeLastObject];

        if ([_modalWindows count] == 0)
        {
            [_modalWindows release];
            _modalWindows = nil;
        }
    }
}

static inline NSWindow *lastModalWindow()
{
    return [_modalWindows lastObject];
}


- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self
        name:NSWindowDidBecomeKeyNotification object:nil];
    [super dealloc];
}

- (NSModalSession)beginModalSessionForWindow:(NSWindow *)aWindow
{
    addModalWindow(aWindow);
    return [super beginModalSessionForWindow:aWindow];
}

- (void)endModalSession:(NSModalSession)session
{
    removeLastModalWindow();
    [super endModalSession:session];
}

/*
- (int)runModalSession:(NSModalSession)session
{
    return [super runModalSession:session];
}
*/

- (void)modalPatchWindowDidBecomeKey:(NSNotification *)aNotification
{
    NSWindow *theKeyWindow = [aNotification object];

    if ([self _isRunningModal])
    {
        NSWindow *theModalWindow = lastModalWindow();

        if (theModalWindow && theKeyWindow != theModalWindow)
        {
            [theModalWindow makeKeyAndOrderFront:self];
        }
    }
/*
    else
    {
        // [TRH] I hope this will activate the main window after closing
        //  a non-main panel. (Does not seem to work, though...)
///        if ([theKeyWindow isMainWindow])
        {
#if DEBUG
///            NSLog(@"ModalPatch -- trying to force main window to front");
#endif
            ///[theKeyWindow orderFront:self];
            ///[theKeyWindow orderFrontRegardless]; // More agressive alternative.
            [self activateIgnoringOtherApps:YES]; // even more agressive...
        }
    }
*/
}
 
/* If your application contains floating panels, it is easy to lose a
   modal panel.  Patch by St\'ephane Corth\'esy <stephane@lysis.ch> */

- _orderFrontModalWindow:(NSWindow *)aWindow
{
    [aWindow setLevel:NSFloatingWindowLevel + 1];
    return [super _orderFrontModalWindow:aWindow];
} 

#endif // MODAL_PATCH

@end // ModalPatchApplication
